function bbox = rgbdbbox(score, pfea, model, overlap)

if nargin < 4
   overlap = 0.5;
end

for i = 1:length(model)
    onebbox = getbbox(i, score, pfea, model);
    if length(onebbox)
       pick = nms(onebbox, overlap);
       onebbox = onebbox(pick,:);

       % Keep only topN
       if size(onebbox,1) > model{i}.topN
          disp(sprintf('%s_%d: keeping topN',model{i}.category,model{i}.instance));
          [sorted I] = sort(onebbox(:,5),'descend');
          onebbox = onebbox(I(1:model{i}.topN),:);
       end

       bboxcell{i} = onebbox;
    else
       bboxcell{i} = [];
    end
end

bbox = nmscategory(bboxcell, overlap); % return bboxes obtained by category detector

if length(bbox)
   bbox = bboxstructure(bbox, score, model);
else
   bbox = [];
end

function bbox = bboxstructure(bboxmatrix, score, model)

for i = 1:size(bboxmatrix,1)
    bbox(i).category = model{bboxmatrix(i,9)}.category;
    bbox(i).instance = bboxmatrix(i,10);
    bbox(i).left = bboxmatrix(i,1);
    bbox(i).right = bboxmatrix(i,3);
    bbox(i).top = bboxmatrix(i,2);
    bbox(i).bottom = bboxmatrix(i,4);
    bbox(i).score = bboxmatrix(i,5);
    bbox(i).scale = bboxmatrix(i,6);
    bbox(i).cheight = bboxmatrix(i,7);
    bbox(i).cwidth = bboxmatrix(i,8);
    
    pixdims = [bbox(i).bottom-bbox(i).top+1 bbox(i).right-bbox(i).left+1];
    modelnum = bboxmatrix(i,9);
    grid = model{modelnum}.grid/bbox(i).scale;
    hInc = round((bbox(i).cheight-bbox(i).top-grid/2)/grid);
    wInc = round((bbox(i).cwidth-bbox(i).left-grid/2)/grid);
    top = bboxmatrix(i,12);
    left = bboxmatrix(i,13);
    scdims = size(score{bboxmatrix(i,9),bboxmatrix(i,11)});
    sc = nan(scdims(1)+2*hInc,scdims(2)+2*wInc);
    sc(hInc+(1:scdims(1)),wInc+(1:scdims(2))) = score{bboxmatrix(i,9),bboxmatrix(i,11)};
    bbox(i).pixScores = imresize(sc(hInc+((top-hInc):(top+hInc)),wInc+((left-wInc):(left+wInc))),[pixdims(1) pixdims(2)],'nearest');
end


function bbox = nmscategory(bboxcell, overlap)

bbox = [];
for i = 1:length(bboxcell)
    bbox = [bbox; bboxcell{i}];
end
pick = nms(bbox, overlap);
bbox = bbox(pick,:);

function bbox = nmsinstance(bboxcell, overlap)

bbox = [];
for i = 1:length(bboxcell)
    bbox = [bbox; bboxcell{i}];
end
pick = nmsi(bbox, overlap);
bbox = bbox(pick,:);

function bbox = getbbox(modelnum, score, pfea, model)

grid = model{modelnum}.grid;
fheight = model{modelnum}.fheight;
fwidth = model{modelnum}.fwidth;
height = model{modelnum}.height;
width = model{modelnum}.width;

it = 0;
for i = 1:size(score,2)
    mscore(i,1) = max(score{modelnum,i}(:));
    if mscore(i,1) > model{modelnum}.threshold;
       [mtop, mleft] = find(score{modelnum,i} > model{modelnum}.threshold);
       for j = 1:length(mtop)
           it = it + 1;
           % compute bounding boxes
           bbox(it,9) = modelnum; % categroy
           bbox(it,10) = model{modelnum}.instance; % instance
           bbox(it,11:13) = [i mtop(j) mleft(j)];
           bbox(it,5) = score{modelnum,i}(mtop(j), mleft(j)); % score
           bbox(it,6) = pfea.scale(i); % scale
           bbox(it,2) = max(round(grid/pfea.scale(i)*(mtop(j)-1))+1,1); % top
           bbox(it,1) = max(round(grid/pfea.scale(i)*(mleft(j)-1))+1,1); % left
           bbox(it,4) = min(round(bbox(it,2) + grid/pfea.scale(i)*(fheight + 2)), pfea.imsize(1)); % bottom
           bbox(it,3) = min(round(bbox(it,1) + grid/pfea.scale(i)*(fwidth + 2)), pfea.imsize(2)); % right
           bbox(it,7) = round((bbox(it,2) + (bbox(it,4) - bbox(it,2))/2.0)); % height center
           bbox(it,8) = round((bbox(it,1) + (bbox(it,3) - bbox(it,1))/2.0)); % width center
       end
    end
end 

if max(mscore) <= model{modelnum}.threshold
   bbox = [];
   return;
end

% Prune based on scale
if false %isfield(model{modelnum},'depthScale') && size(bbox,1) > 0
   if strcmp(model{modelnum}.category,'cap')
      disp('');
   end
   keeps = true(1,size(bbox,1));
   for b=1:size(bbox)
      center = [(bbox(b,2)+bbox(b,4))/2 (bbox(b,1)+bbox(b,3))/2];
      d = model{modelnum}.depth(round(center(1)-model{modelnum}.grid/bbox(b,6)/2):round(center(1)+model{modelnum}.grid/bbox(b,6)/2),round(center(2)-model{modelnum}.grid/bbox(b,6)/2):round(center(2)+model{modelnum}.grid/bbox(b,6)/2));
      d = double(median(d(:)));
      keep = false;
      [predScale stdev] = polyval(model{modelnum}.depthScale.p,d,model{modelnum}.depthScale.s);
      if min([predScale bbox(b,6)])/max([predScale bbox(b,6)]) > 0.6
%      if abs(predScale-bbox(b,6)) < 0.5*stdev
         keep = true; break;
      end
      keeps(b) = keep;
   end
   bbox = bbox(keeps,:);
end


function pick = nmsi(boxes, overlap)
% pick = nmsi(boxes, overlap)
% non-maximum suppression.
% Greedily select high-scoring detections and skip detections
% that are significantly covered by a previously selected detection.

if isempty(boxes)
  pick = [];
else
  x1 = boxes(:,1);
  y1 = boxes(:,2);
  x2 = boxes(:,3);
  y2 = boxes(:,4);
  s = boxes(:,5);
  area = (x2-x1+1) .* (y2-y1+1);

  [vals, I] = sort(s);
  pick = [];
  while ~isempty(I)
    last = length(I);
    i = I(last);
    pick = [pick; i];
    suppress = [last];
    for pos = 1:last-1
      j = I(pos);
      xx1 = max(x1(i), x1(j));
      yy1 = max(y1(i), y1(j));
      xx2 = min(x2(i), x2(j));
      yy2 = min(y2(i), y2(j));
      w = xx2-xx1+1;
      h = yy2-yy1+1;
      if w > 0 && h > 0
        % compute overlap
        o = w * h / area(j);
        if o > overlap
          suppress = [suppress; pos];
        end
      end
      tag = boxes(i,9) == boxes(j,9) | boxes(i,10) == boxes(j,10);
      if tag & suppress(end) ~= pos
         suppress = [suppress; pos];
      end
    end
    I(suppress) = [];
  end
end


function pick = nms(boxes, overlap)
% pick = nms(boxes, overlap)
% non-maximum suppression.
% Greedily select high-scoring detections and skip detections
% that are significantly covered by a previously selected detection.

if isempty(boxes)
  pick = [];
else
  x1 = boxes(:,1);
  y1 = boxes(:,2);
  x2 = boxes(:,3);
  y2 = boxes(:,4);
  s = boxes(:,5);
  area = (x2-x1+1) .* (y2-y1+1);

  [vals, I] = sort(s);
  pick = [];
  while ~isempty(I)
    last = length(I);
    i = I(last);
    pick = [pick; i];
    suppress = [last];
    for pos = 1:last-1
      j = I(pos);
      xx1 = max(x1(i), x1(j));
      yy1 = max(y1(i), y1(j));
      xx2 = min(x2(i), x2(j));
      yy2 = min(y2(i), y2(j));
      w = xx2-xx1+1;
      h = yy2-yy1+1;
      if w > 0 && h > 0
        % compute overlap
        o = w * h / area(j);
        if o > overlap
          suppress = [suppress; pos];
        end
      end
    end
    I(suppress) = [];
  end
end


