function bbox = rgbdbbox(score, pfea, model, overlap)

if nargin < 4
   overlap = 0.5;
end

for i = 1:length(model)
    onebbox = getbbox(i, score, pfea, model);
    if length(onebbox)
       pick = nms(onebbox, overlap);
       onebbox = onebbox(pick,:);
       bboxcell{i} = onebbox;
    else
       bboxcell{i} = [];
    end
end

if model{1}.instance > 0
   bbox = nmsinstance(bboxcell, overlap); % return bboxes obtained by instance detector
else
   bbox = nmscategory(bboxcell, overlap); % return bboxes obtained by category detector
end

if length(bbox)
   bbox = bboxstructure(bbox, model);
else
   bbox = [];
end

function bbox = bboxstructure(bboxmatrix, model)

for i = 1:size(bboxmatrix,1)
    bbox(i).category = model{bboxmatrix(i,9)}.category;
    bbox(i).instance = bboxmatrix(i,10);
    bbox(i).left = bboxmatrix(i,1);
    bbox(i).right = bboxmatrix(i,3);
    bbox(i).top = bboxmatrix(i,2);
    bbox(i).bottom = bboxmatrix(i,4);
    bbox(i).score = bboxmatrix(i,5);
    bbox(i).scale = bboxmatrix(i,6);
    bbox(i).cheight = bboxmatrix(i,7);
    bbox(i).cwidth = bboxmatrix(i,8);
end


function bbox = nmscategory(bboxcell, overlap)

bbox = [];
for i = 1:length(bboxcell)
    bbox = [bbox; bboxcell{i}];
end
pick = nms(bbox, overlap);
bbox = bbox(pick,:);

function bbox = nmsinstance(bboxcell, overlap)

bbox = [];
for i = 1:length(bboxcell)
    bbox = [bbox; bboxcell{i}];
end
pick = nmsi(bbox, overlap);
bbox = bbox(pick,:);

function bbox = getbbox(modelnum, score, pfea, model)

grid = model{modelnum}.grid;
fheight = model{modelnum}.fheight;
fwidth = model{modelnum}.fwidth;
height = model{modelnum}.height;
width = model{modelnum}.width;

it = 0;
for i = 1:size(score,2)
    mscore(i,1) = max(score{modelnum,i}(:));
    if mscore(i,1) > model{modelnum}.threshold;
       [mtop, mleft] = find(score{modelnum,i} > model{modelnum}.threshold);
       for j = 1:length(mtop)
           it = it + 1;
           % compute bounding boxes
           bbox(it,9) = modelnum; % categroy
           bbox(it,10) = model{modelnum}.instance; % instance
           bbox(it,5) = score{modelnum,i}(mtop(j), mleft(j)); % score
           bbox(it,6) = pfea.scale(i); % scale
           bbox(it,2) = max(round(grid/pfea.scale(i)*mtop(j))+1,1); % top
           bbox(it,1) = max(round(grid/pfea.scale(i)*mleft(j))+1,1); % left
           bbox(it,4) = min(round(bbox(it,2) + grid/pfea.scale(i)*(fheight + 2)), pfea.imsize(1)); % bottom
           bbox(it,3) = min(round(bbox(it,1) + grid/pfea.scale(i)*(fwidth + 2)), pfea.imsize(2)); % right
           bbox(it,7) = round((bbox(it,2) + (bbox(it,4) - bbox(it,2))/2.0)); % height center
           bbox(it,8) = round((bbox(it,1) + (bbox(it,3) - bbox(it,1))/2.0)); % width center
       end
    end
end 
if max(mscore) <= model{modelnum}.threshold
   bbox = [];
end

function pick = nmsi(boxes, overlap)
% pick = nmsi(boxes, overlap)
% non-maximum suppression.
% Greedily select high-scoring detections and skip detections
% that are significantly covered by a previously selected detection.

if isempty(boxes)
  pick = [];
else
  x1 = boxes(:,1);
  y1 = boxes(:,2);
  x2 = boxes(:,3);
  y2 = boxes(:,4);
  s = boxes(:,5);
  area = (x2-x1+1) .* (y2-y1+1);

  [vals, I] = sort(s);
  pick = [];
  while ~isempty(I)
    last = length(I);
    i = I(last);
    pick = [pick; i];
    suppress = [last];
    for pos = 1:last-1
      j = I(pos);
      xx1 = max(x1(i), x1(j));
      yy1 = max(y1(i), y1(j));
      xx2 = min(x2(i), x2(j));
      yy2 = min(y2(i), y2(j));
      w = xx2-xx1+1;
      h = yy2-yy1+1;
      if w > 0 && h > 0
        % compute overlap
        o = w * h / area(j);
        if o > overlap
          suppress = [suppress; pos];
        end
      end
      tag = boxes(i,9) == boxes(j,9) | boxes(i,10) == boxes(j,10);
      if tag & suppress(end) ~= pos
         suppress = [suppress; pos];
      end
    end
    I(suppress) = [];
  end
end


function pick = nms(boxes, overlap)
% pick = nms(boxes, overlap)
% non-maximum suppression.
% Greedily select high-scoring detections and skip detections
% that are significantly covered by a previously selected detection.

if isempty(boxes)
  pick = [];
else
  x1 = boxes(:,1);
  y1 = boxes(:,2);
  x2 = boxes(:,3);
  y2 = boxes(:,4);
  s = boxes(:,5);
  area = (x2-x1+1) .* (y2-y1+1);

  [vals, I] = sort(s);
  pick = [];
  while ~isempty(I)
    last = length(I);
    i = I(last);
    pick = [pick; i];
    suppress = [last];
    for pos = 1:last-1
      j = I(pos);
      xx1 = max(x1(i), x1(j));
      yy1 = max(y1(i), y1(j));
      xx2 = min(x2(i), x2(j));
      yy2 = min(y2(i), y2(j));
      w = xx2-xx1+1;
      h = yy2-yy1+1;
      if w > 0 && h > 0
        % compute overlap
        o = w * h / area(j);
        if o > overlap
          suppress = [suppress; pos];
        end
      end
    end
    I(suppress) = [];
  end
end


